@@ -5,14 +5,15 @@ document.addEventListener('DOMContentLoaded', function () {
 
   (function tweakSidebar() { try {
     var donateH3 = sidebar.querySelector('.donation');
-    donateH3.textContent = 'Giving back';
+    donateH3.textContent = 'Sponsor';
     var tideliftP = donateH3.nextElementSibling.nextElementSibling;
-    var givebackA = document.createElement('a');
-    givebackA.href = 'https://gumroad.com/l/bidict';
-    givebackA.textContent = 'Bidict is the product of hundreds of hours of unpaid, voluntary work. If bidict has helped you accomplish your work, click here to chip in toward the costs of bidict’s maintenance and development.'
-    sidebar.insertBefore(givebackA, tideliftP);
+    var sponsorA = document.createElement('a');
+    sponsorA.href = 'https://github.com/sponsors/jab';
+    sponsorA.textContent = 'Sponsor bidict on GitHub.'
+    sidebar.insertBefore(sponsorA, tideliftP);
     var tideliftH3 = document.createElement('h3');
-    tideliftH3.textContent = 'Get Support';
+    tideliftH3.textContent = 'Enterprise';
+    tideliftP.innerHTML = 'Enterprise support for bidict is available <a href="https://tidelift.com/subscription/pkg/pypi-bidict?utm_source=pypi-bidict&utm_medium=referral&utm_campaign=docs">via Tidelift</a>.';
     sidebar.insertBefore(tideliftH3, tideliftP);
    } catch (e) {}
   })();
@@ -31,9 +32,6 @@ document.addEventListener('DOMContentLoaded', function () {
     document.body.append(script);
   }
 
-  addDiv('id', 'codefund');
-  addScript('https://codefund.app/properties/179/funder.js');
-
   addDiv('className', 'rc-scout');
   addScript('https://www.recurse-scout.com/loader.js?t=c17a917136a40c38f5ce6b80adbbfd19');
 });
