@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2009-2020 Joshua Bronson. All Rights Reserved.
+# Copyright 2009-2021 Joshua Bronson. All Rights Reserved.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -45,7 +45,7 @@ https://bidict.readthedocs.io for the most up-to-date documentation
 if you are reading this elsewhere.
 
 
-.. :copyright: (c) 2009-2020 Joshua Bronson.
+.. :copyright: (c) 2009-2021 Joshua Bronson.
 .. :license: MPLv2. See LICENSE for details.
 """
 
@@ -56,9 +56,6 @@ from sys import version_info as _version_info
 if _version_info < (3, 6):  # pragma: no cover
     raise ImportError('Python 3.6+ is required.')
 
-# The rest of this file only collects functionality implemented in the rest of the
-# source for the purposes of exporting it under the `bidict` module namespace.
-# flake8: noqa: F401 (imported but unused)
 from ._abc import BidirectionalMapping, MutableBidirectionalMapping
 from ._base import BidictBase
 from ._mut import MutableBidict
@@ -73,18 +70,18 @@ from ._exc import BidictException, DuplicationError, KeyDuplicationError, ValueD
 from ._iter import inverted
 from .metadata import (
     __author__, __maintainer__, __copyright__, __email__, __credits__, __url__,
-    __license__, __status__, __description__, __keywords__, __version__, __version_info__,
+    __license__, __status__, __description__, __keywords__, __version__,
 )
 
 # Set __module__ of re-exported classes to the 'bidict' top-level module name
-# so that private/internal submodules are not exposed to users e.g. in repr  strings.
+# so that private/internal submodules are not exposed to users e.g. in repr strings.
 _locals = tuple(locals().items())
 for _name, _obj in _locals:  # pragma: no cover
     if not getattr(_obj, '__module__', '').startswith('bidict.'):
         continue
     try:
         _obj.__module__ = 'bidict'
-    except AttributeError as exc:  # raised when __module__ is read-only (as in OnDup)
+    except AttributeError:  # raised when __module__ is read-only (as in OnDup)
         pass
 
 
