@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2009-2020 Joshua Bronson. All Rights Reserved.
+# Copyright 2009-2021 Joshua Bronson. All Rights Reserved.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -18,13 +18,14 @@ from platform import python_implementation
 from weakref import ref
 
 import pytest
-from hypothesis import example, given
+from hypothesis import assume, example, given
 
 from bidict import (
     BidictException,
     DROP_OLD, RAISE, OnDup,
     OrderedBidictBase, OrderedBidict, bidict, namedbidict,
     inverted,
+    KeyDuplicationError, ValueDuplicationError, KeyAndValueDuplicationError,
 )
 from bidict._iter import _iteritems_args_kw
 
@@ -54,7 +55,7 @@ def test_unequal_to_mapping_with_different_items(bi_and_map_from_diff_items):
     assert not bi == mapping
 
 
-@given(st.BI_AND_MAP_FROM_SAME_ITEMS)
+@given(st.BI_AND_MAP_FROM_SAME_ND_ITEMS)
 def test_equal_to_mapping_with_same_items(bi_and_map_from_same_items):
     """Bidicts should be equal to mappings created from the same non-duplicating items.
 
@@ -68,7 +69,7 @@ def test_equal_to_mapping_with_same_items(bi_and_map_from_same_items):
     assert not bi.inv != mapping_inv
 
 
-@given(st.HBI_AND_HMAP_FROM_SAME_ITEMS)
+@given(st.HBI_AND_HMAP_FROM_SAME_ND_ITEMS)
 def test_equal_hashables_have_same_hash(hashable_bidict_and_mapping):
     """Hashable bidicts and hashable mappings that are equal should hash to the same value."""
     bi, mapping = hashable_bidict_and_mapping
@@ -76,16 +77,16 @@ def test_equal_hashables_have_same_hash(hashable_bidict_and_mapping):
     assert hash(bi) == hash(mapping)
 
 
-@given(st.OBI_AND_OMAP_FROM_SAME_ITEMS)
-def test_equals_order_sensitive(ob_and_om):
+@given(st.BI_AND_MAP_FROM_SAME_ND_ITEMS)
+def test_equals_order_sensitive(bi_and_map_from_same_items):
     """Ordered bidicts should be order-sensitive-equal to ordered mappings with same nondup items.
 
     The bidict's inverse and the ordered mapping's inverse should also be order-sensitive-equal.
     """
-    ob, om = ob_and_om
-    assert ob.equals_order_sensitive(om)
-    om_inv = OrderedDict((v, k) for (k, v) in om.items())
-    assert ob.inv.equals_order_sensitive(om_inv)
+    bi, mapping = bi_and_map_from_same_items
+    assert bi.equals_order_sensitive(mapping)
+    mapping_inv = {v: k for (k, v) in mapping.items()}
+    assert bi.inv.equals_order_sensitive(mapping_inv)
 
 
 @given(st.OBI_AND_OMAP_FROM_SAME_ITEMS_DIFF_ORDER)
@@ -116,6 +117,44 @@ def test_unequal_order_sensitive_non_mapping(ob, not_a_mapping):
     assert not ob.inv.equals_order_sensitive(not_a_mapping)
 
 
+@given(st.MUTABLE_BIDICTS, st.DIFF_ATOMS, st.RANDOMS)
+def test_setitem_with_dup_val_raises(bi, new_key, rand):
+    """Setting an item whose value duplicates that of an existing item should raise ValueDuplicationError."""
+    ln = len(bi)
+    assume(ln > 2)
+    for b in (bi, bi.inv):
+        existing_val = rand.choice(list(b.inv))
+        with pytest.raises(ValueDuplicationError):
+            b[new_key] = existing_val
+        assert len(b) == len(b.inv) == ln
+
+
+@given(st.MUTABLE_BIDICTS, st.RANDOMS)
+def test_setitem_with_dup_key_val_raises(bi, rand):
+    """Setting an item whose key and val duplicate two different existing items raises KeyAndValueDuplicationError."""
+    ln = len(bi)
+    assume(ln > 2)
+    for b in (bi, bi.inv):
+        existing_items = rand.sample(list(b.items()), 2)
+        existing_key = existing_items[0][0]
+        existing_val = existing_items[1][1]
+        with pytest.raises(KeyAndValueDuplicationError):
+            b[existing_key] = existing_val
+        assert len(b) == len(b.inv) == ln
+
+
+@given(st.MUTABLE_BIDICTS, st.DIFF_ATOMS, st.RANDOMS)
+def test_put_with_dup_key_raises(bi, new_val, rand):
+    """Putting an item whose key duplicates that of an existing item should raise KeyDuplicationError."""
+    ln = len(bi)
+    assume(ln > 2)
+    for b in (bi, bi.inv):
+        existing_key = rand.choice(list(b))
+        with pytest.raises(KeyDuplicationError):
+            b.put(existing_key, new_val)
+        assert len(b) == len(b.inv) == ln
+
+
 @given(st.BIDICTS)
 def test_bijectivity(bi):
     """b[k] == v  <==>  b.inv[v] == k"""
@@ -123,6 +162,13 @@ def test_bijectivity(bi):
         assert all(b.inv[v] == k for (k, v) in b.items())
 
 
+@given(st.MUTABLE_BIDICTS)
+def test_cleared_bidicts_have_no_items(bi):
+    bi.clear()
+    assert not bi
+    assert len(bi) == 0
+
+
 @given(st.BI_AND_CMPDICT_FROM_SAME_ITEMS, st.ARGS_BY_METHOD)
 def test_consistency_after_method_call(bi_and_cmp_dict, args_by_method):
     """A bidict should be left in a consistent state after calling any method, even if it raises."""
@@ -190,27 +236,18 @@ def test_putall_same_as_put_for_each_item(bi, items, on_dup):
     assert check.inv == expect.inv
 
 
-@given(st.BI_AND_CMPDICT_FROM_SAME_ITEMS)
-def test_iter(bi_and_cmp_dict):
-    """:meth:`bidict.BidictBase.__iter__` should yield all the keys in a bidict."""
-    bi, cmp_dict = bi_and_cmp_dict
-    assert set(bi) == cmp_dict.keys()
+@given(st.BI_AND_MAP_FROM_SAME_ND_ITEMS)
+def test_bidict_iter(bi_and_mapping):
+    """iter(bi) should yield the keys in a bidict in insertion order."""
+    bi, mapping = bi_and_mapping
+    assert all(i == j for (i, j) in zip(bi, mapping))
 
 
-@given(st.OBI_AND_OD_FROM_SAME_ITEMS)
-def test_orderedbidict_iter(ob_and_od):
-    """Ordered bidict __iter__ should yield all the keys in an ordered bidict in the right order."""
-    ob, od = ob_and_od
-    assert all(i == j for (i, j) in zip(ob, od))
-
-
-@given(st.OBI_AND_OD_FROM_SAME_ITEMS)
-def test_orderedbidict_reversed(ob_and_od):
-    """:meth:`bidict.OrderedBidictBase.__reversed__` should yield all the keys
-    in an ordered bidict in the reverse-order they were inserted.
-    """
-    ob, od = ob_and_od
-    assert all(i == j for (i, j) in zip(reversed(ob), reversed(od)))
+@given(st.RBI_AND_RMAP_FROM_SAME_ND_ITEMS)
+def test_bidict_reversed(rb_and_rd):
+    """reversed(bi) should yield the keys in a bidict in reverse insertion order."""
+    rb, rd = rb_and_rd
+    assert all(i == j for (i, j) in zip(reversed(rb), reversed(rd)))
 
 
 @given(st.FROZEN_BIDICTS)
@@ -381,19 +418,10 @@ def test_inverted_pairs(pairs):
     assert list(inverted(inverted(pairs))) == pairs
 
 
-@given(st.BI_AND_CMPDICT_FROM_SAME_ITEMS)
-def test_inverted_bidict(bi_and_cmp_dict):
-    """:func:`bidict.inverted` should yield the inverse items of a bidict."""
-    bi, cmp_dict = bi_and_cmp_dict
-    cmp_dict_inv = OrderedDict((v, k) for (k, v) in cmp_dict.items())
-    assert set(inverted(bi)) == cmp_dict_inv.items() == bi.inv.items()
-    assert set(inverted(inverted(bi))) == cmp_dict.items() == bi.inv.inv.items()
-
-
-@given(st.OBI_AND_OD_FROM_SAME_ITEMS)
-def test_inverted_orderedbidict(ob_and_od):
+@given(st.BI_AND_MAP_FROM_SAME_ND_ITEMS)
+def test_inverted_bidict(bi_and_mapping):
     """:func:`bidict.inverted` should yield the inverse items of an ordered bidict."""
-    ob, od = ob_and_od
-    od_inv = OrderedDict((v, k) for (k, v) in od.items())
-    assert all(i == j for (i, j) in zip(inverted(ob), od_inv.items()))
-    assert all(i == j for (i, j) in zip(inverted(inverted(ob)), od.items()))
+    bi, mapping = bi_and_mapping
+    mapping_inv = {v: k for (k, v) in mapping.items()}
+    assert all(i == j for (i, j) in zip(inverted(bi), mapping_inv.items()))
+    assert all(i == j for (i, j) in zip(inverted(inverted(bi)), mapping.items()))
